#!/bin/sh

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="drwatson[$$]"

#
# send_mail
#
#    $1    To Field
#    $2    Subject Field
#    $3    Body
#
send_mail()
{
  SENDMAIL_TO=""
  RECIPIENT_TO=""
  for to in $1 ; do
    if [ -z "$SENDMAIL_TO" ] ; then
      SENDMAIL_TO="$to"
      RECIPIENT_TO="$to"
    else
      SENDMAIL_TO="$SENDMAIL_TO $to"
      RECIPIENT_TO="$RECIPIENT_TO, $to"
    fi
  done

HOSTNAME=`hostname`

   /usr/sbin/sendmail $SENDMAIL_TO  <<EndWatsonMail
From: Watson <watson@${HOSTNAME}>
To: $RECIPIENT_TO
Subject: $2 (${HOSTNAME})

$(cat "$3" 2>/dev/null)
.
EndWatsonMail

}


#
# Sends a message file 
#
#    $1    To Field
#    $2    message type (Error, Warning, Kernel error, etc)
#    $3    message file (WILL GET DELETED!!!)

send_msg_file()
{
if [ -f "$3" ] ; then
  MSG_TYPE=$2
  MSG_TYPE_L=`echo $2 | tr [A-Z] [a-z]`

  mv $3 $3.tmp
  if [ `grep -c "\n" $3.tmp` -eq 1 ] ; then

    cat $3.tmp | \
    while read MSG ; do
      $LOGGER -p $LOG_FACILITY.err -t "$TAG" "$MSG_TYPE: $MSG [send mail to: $1]"
      [ -n "${1}" ] && send_mail "${1}" "$MSG_TYPE: $MSG" ""
    done
  else
    $LOGGER -p $LOG_FACILITY.err -t "$TAG" "Multiple ${MSG_TYPE_L} messages [send mail to: $1]"
    [ -n "${1}" ] && send_mail "${1}" "Multiple ${MSG_TYPE_L} messages" $3.tmp
  fi
  rm $3.tmp
fi
}

#
# Checks for kernel error/warning messages
#
# $1 to
# $2 Error|Warning
#
check_kernel_messages()
{
  MSG_TYPE="$2"
  MESSAGE_TYPE=`echo $MSG_TYPE | tr [A-Z] [a-z]`
  MESSAGE_TO="$1"
  MSG_FILE_NAME="kernel_${MESSAGE_TYPE}_msg"
  TMP_FILE_NAME="${TMP}/${MSG_FILE_NAME}"
  VAR_FILE_NAME="${DRWATSON_VAR}/${MSG_FILE_NAME}"

  $ASROOT lastlogs.asr 500 "${MESSAGE_TYPE}s" | grep -vF 'sshd(pam_unix)' > ${TMP_FILE_NAME}

  [ ! -f "${VAR_FILE_NAME}" ] && touch ${VAR_FILE_NAME}

  cmp -s ${TMP_FILE_NAME} ${VAR_FILE_NAME}

  if [ $? -ne 0 ] ; then
    diff ${VAR_FILE_NAME} ${TMP_FILE_NAME} | sed -n 's;^> ;;p' > ${TMP_FILE_NAME}.diff
    mv ${TMP_FILE_NAME} ${VAR_FILE_NAME}

    send_msg_file ${MESSAGE_TO} "Kernel ${MESSAGE_TYPE}" ${TMP_FILE_NAME}.diff
  else
    rm -f ${TMP_FILE_NAME}
  fi
}

# Recipients

ERROR_TO="${DRWATSON_ERROR_MAIL}"
[ -f ${DRWATSON_CONFIG}/error_mail ] && ERROR_TO="`cat ${DRWATSON_CONFIG}/error_mail`"

WARNING_TO="${DRWATSON_WARNING_MAIL}"
[ -f ${DRWATSON_CONFIG}/warning_mail ] && WARNING_TO="`cat ${DRWATSON_CONFIG}/warning_mail`"

READY_TO="${DRWATSON_READY_MAIL}"
[ -f ${DRWATSON_CONFIG}/ready_mail ] && READY_TO="`cat ${DRWATSON_CONFIG}/ready_mail`"

# Check Kernel Messages

check_kernel_messages "${ERROR_TO}"   Error
check_kernel_messages "${WARNING_TO}" Warning

# check general errors

send_msg_file ${ERROR_TO}   "Error"   $DR_WATSON_ERROR_FILE
send_msg_file ${WARNING_TO} "Warning" $DR_WATSON_WARNING_FILE
send_msg_file ${READY_TO}   "Ready"   $DR_WATSON_READY_FILE


# Check components

[ ! -f "$DRWATSON_VAR/run" ] && exit

COMPONENT_DIR=$HOME/component

$LOGGER -p $LOG_FACILITY.info -t "$TAG" "check components started"
for i in ${COMPONENTS_S}; do
  if [ -x ${COMPONENT_DIR}/$i/ctrl ] ; then
    ${COMPONENT_DIR}/$i/ctrl status > $TMP/drwatson.$i.$$ &
    timeout_pid=$!

    #handle timeout
    let timeout_i=1

    while [ $timeout_i -lt 30 ] ; do
      [ ! -e /proc/$timeout_pid ] && break
      sleep 1
      let timeout_i=timeout_i+1
    done

    # kill if need
    if [ -e /proc/$timeout_pid ] ; then
      kill -9 $timeout_pid
      STAT="ERROR"
    else
      STAT=`cat $TMP/drwatson.$i.$$`
    fi

    rm $TMP/drwatson.$i.$$ 2>/dev/null

  else
    STAT="NOT AVAILABLE"
  fi

  if [ "$STAT" = "ERROR" -o "$STAT" = "STOPPED" ] ; then
    if [ ! -f $DRWATSON_FILES/$i.error ] ; then
      date=`date "+%Y/%m/%d %H:%M:%S"`
      echo "1" > $DRWATSON_FILES/$i.error

      $LOGGER -p $LOG_FACILITY.err -t "$TAG" "$i $STAT [send mail to: $ERROR_TO]"
      send_mail "${ERROR_TO}" "$STAT: $i" ""
    else
      $LOGGER -p $LOG_FACILITY.err -t "$TAG" "$i STILL$STAT"
      if [ "$STAT" = "ERROR" ] ; then
        num=`cat $DRWATSON_FILES/$i.error`;
        if [ "$num" -eq 3 ] ; then
          ${COMPONENT_DIR}/$i/ctrl emergency
        fi
        num1=`expr $num + 1`
        echo "$num1" > $DRWATSON_FILES/$i.error
      fi
    fi
  else
    if [ -f $DRWATSON_FILES/$i.error ] ; then
      $LOGGER -p $LOG_FACILITY.err -t "$TAG" "$i READY [send mail to: $READY_TO]"
      send_mail "${READY_TO}" "$i READY" ""
      rm $DRWATSON_FILES/$i.error 2>/dev/null;
    fi
  fi
done
$LOGGER -p $LOG_FACILITY.info -t "$TAG" "check components finished"
