#!/bin/sh

# Component: drwatson
#
# Description:
#  Getting component status and monitoring info for Siteman
#
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$

. $HOME/etc/setenv.sh

echo "statusValuesComing"

#Evaluating Monitor Status

defaultStatusValuesFile=""
[ -f "$DRWATSON_HOME/monitorStatusValues.cfg" ] && defaultStatusValuesFile="$DRWATSON_HOME/monitorStatusValues.cfg"

statusValuesFile=""
[ -f "$DRWATSON_CONFIG/monitorStatusValues.cfg" ] && statusValuesFile="$DRWATSON_CONFIG/monitorStatusValues.cfg"

cat $defaultStatusValuesFile $statusValuesFile $LOG/monitor.log | grep "=" | grep -v "#" | LC_NUMERIC=en_US awk -vFS="=|-" -vOFS="=" '

#valuesLoaded will be true when monitor.log begins
	$1 == "monitorSTART" {
		valuesLoaded = 1;
	}
	valuesLoaded == 0 {
		limit0Array[ $1 ] = $2;	
		if( NF >= 3 ) {
			limit1Array[ $1 ] = $3;		
		}
	}
#timeOK will be true when time is over than NOW - 1 hour (3600 sec)
	timeOK == 0 && $1 == "monitorSTART" { 
		monTime = mktime( gensub( "[/:]", " ", "g", $2 ) ); 
		if( monTime >= systime() - 3600 ) {
			timeOK = 1;
		}
	} 
	timeOK == 1 && $1 == "monitorSTART" {
		monTime = mktime( gensub( "[/:]", " ", "g", $2 ) ); 
		print( "monitorRelTime", systime() - monTime );
	}
	timeOK == 1 && $1 != "monitorSTART" {
		monitorStatus = 0;
		if( $1 in limit0Array ) {
			if( !( $1 in limit1Array ) ) {
				if( $2 >= limit0Array[ $1 ] ) {
					monitorStatus = 10;
				}
			}
			else {
				if( $2 < limit0Array[ $1 ] ) {
					monitorStatus = 0;
				}
				else if( $2 > limit1Array[ $1 ] ) {
					monitorStatus = 10;
				}
				else if( limit0Array[ $1 ] < limit1Array[ $1 ] ) {
					monitorStatus = int( ( $2 - limit0Array[ $1 ] ) * 10 / ( limit1Array[ $1 ] - limit0Array[ $1 ] ) + 0.5 );
				}
				else {
					monitorStatus = 10;
				}
			}
		}
		if( monitorStatus != 0 ) {
			print( $1, monitorStatus );
		}
	}
' 


#Evaluating Error/Stopped status
#within 1 hour; but between 0-1 am use previous day '23 hours later' too

startDate=`date +%F`
#-d "1 day ago" lenne, de a mai feliratuban vannak a tegnapi adatok

defaultComponentWeightFile=""
[ -f "$DRWATSON_HOME/componentWeight.cfg" ] && defaultComponentWeightFile="$DRWATSON_HOME/componentWeight.cfg"
	
componentWeightFile=""
[ -f "$DRWATSON_CONFIG/componentWeight.cfg" ] && componentWeightFile="$DRWATSON_CONFIG/componentWeight.cfg"

timeOfDay=`date +%T`

(
	[ -n "$componentWeightFile" ] && cat $defaultComponentWeightFile $componentWeightFile
	( 
		ls $LOGS_DATED/week | \
			awk -vFS='.' -vStart=$startDate '($4 == "error" && $2 >= Start) { print $2 " " $0}'  | \
		while read fdate fname; do
			zcat $LOGS_DATED/week/$fname
		done
		echo "drwatsonTODAY"
		cat $LOG/error.log 
	) | grep "drwatson" 
) | LC_NUMERIC=en_US awk -vFS=" " -vOFS="=" -vTIMEOFDAYSTR="$timeOfDay" '
	BEGIN {
		timeofday = substr( TIMEOFDAYSTR, 1, 2 ) * 3600 + substr( TIMEOFDAYSTR, 4, 2 ) * 60 + substr( TIMEOFDAYSTR, 7, 2 );
		#in seconds
		onehourtime = 60 * 60;
	}
	$1 == "drwatsonTODAY" {
		valuesLoaded = 1;
		today = 1;
	}
	match( $5, "drwatson" ) {
		valuesLoaded = 1;
		if( $7 == "STOPPED" || $7 == "ERROR" || $7 == "STILLSTOPPED" || $7 == "STILLERROR" ) {
			messagetime = strtonum( substr( $3, 1, 2 ) ) * 3600 + strtonum( substr( $3, 4, 2 ) ) * 60 + strtonum( substr( $3, 7, 2 ) );
			if( ( ( today == 1 ) &&
				  ( messagetime <= timeofday ) &&
				  ( messagetime >= timeofday - onehourtime ) )     ||
				( ( timeofday < onehourtime ) &&
				  ( messagetime >= timeofday + 23 * onehourtime ) )  )
				{
				if( $7 == "ERROR" || $7 == "STILLERROR" ) {
					status = 10;
				}
				else {
					status = 5;
				}
				if( $6 in componentArray ) {
					status = status * componentArray[ $6 ];
				}
				#output is in seconds, too
				if( ( messagetime <= timeofday ) &&
					( messagetime >= timeofday - onehourtime ) ) {
					print( "statusRelTime", timeofday - messagetime );
				}
				else {
					print( "statusRelTime", timeofday - messagetime + 24 * onehourtime );
				}
				print( $6, status );
			}
		}
	}
	valuesLoaded == 0 {
		componentArray[ $1 ] = $2;
	}
'
