#!/bin/sh

# Component: drwatson
#
# Description:
#  System state monitoring
#
# Usage: monitor.sh [dumpOnly | cron_job] # dumpOnly is default
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$

. $HOME/etc/setenv.sh

usage="Usage: monitor.sh [dumpOnly | cron_job] # dumpOnly is default"
type="dumpOnly"

if [ -n "$1" ] ; then
	if [ "$1" = "-h" -o "$1" = "--help" ] ; then
		echo ; echo "$usage" ; echo
		exit 0
	fi
	type="$1"
fi

if [ "$type" = "dumpOnly" ] ; then
	$ETC/rc monitor
	exit 0
fi


# check monitoring_rate and monitoring_counter at cron job

# monitoring rate configuration - default=1 means : always collect and log monitor info
monitoringRate=1
monitoringRateCfg="$CONF/monitoring_rate.cfg"
[ -f "$monitoringRateCfg" ] && monitoringRate=`cat $monitoringRateCfg`

[ $monitoringRate -eq 0 ] && exit 0

monitoringCounter=0
monitoringCounterFile="$TMP/monitoring_counter"
[ -f "$monitoringCounterFile" ] && monitoringCounter=`cat $monitoringCounterFile`

monitoringCounter=$(( $monitoringCounter + 1 ))

if [ $monitoringCounter -lt $monitoringRate ] ; then
  echo "$monitoringCounter" >$monitoringCounterFile
  exit 0
else	# Will Run!
  echo "0" >$monitoringCounterFile
  sleep 30	# Don't start along with another cron job!
fi

tempLogFile="$TMP/monitor.$$.log.tmp"

startDateTimeLogLine=`date +"monitorSTART=%Y/%m/%d %T"`

echo >$tempLogFile
echo "$startDateTimeLogLine" >>$tempLogFile
echo >>$tempLogFile
$ETC/rc monitor >>$tempLogFile

cat $tempLogFile >>$LOG/monitor.log


### --- Filter by Critical Limits

criticalLimitFile=""
[ -f "$CONF/crucialStage.cfg" ] && criticalLimitFile="$CONF/crucialStage.cfg"
[ -f "$DRWATSON_CONFIG/crucialStage.cfg" ] && criticalLimitFile="$DRWATSON_CONFIG/crucialStage.cfg"
[ -z "$criticalLimitFile" ] && exit 0

criticalValueFile="$TMP/capture.$$.res"

cat $criticalLimitFile $tempLogFile | grep "=" | LC_NUMERIC=en_US awk -vFS="=" '
{
	if ($1 in limitsArray)
	{
		if ( limitsArray[$1] < $2 )
		{
			print $0;
		}
	}
	else
		limitsArray[$1]=$2;
}' >$criticalValueFile
															
criticalValue=`cat $criticalValueFile`
if [ -z "$criticalValue" ] ; then
	rm -f $criticalValueFile
	rm -f $tempLogFile
	cat $criticalLimitFile | grep "=" | awk -vFS="=" '{print $1}' |\
	while read drWatsonFlagFile ; do
		rm -f $TMP/$drWatsonFlagFile
	done
	exit 0
else
	echo "$startDateTimeLogLine" >>$criticalValueFile
fi
																							
$DRWATSON_HOME/capture.sh emergency $criticalValueFile >>$LOG/capture.log 2>&1

rm -f $criticalValueFile
rm -f $tempLogFile